from rest_framework import status

from mayan.apps.documents.permissions import (
    permission_document_type_edit, permission_document_type_view
)
from mayan.apps.documents.tests.mixins.document_mixins import DocumentTestMixin
from mayan.apps.rest_api.tests.base import BaseAPITestCase

from ..events import (
    event_metadata_type_created, event_metadata_type_edited,
    event_metadata_type_relationship_updated
)
from ..models import DocumentTypeMetadataType, MetadataType
from ..permissions import (
    permission_metadata_type_create, permission_metadata_type_delete,
    permission_metadata_type_edit, permission_metadata_type_view
)

from .mixins.document_type_metadata_type_mixins import (
    DocumentTypeMetadataTypeAPIViewTestMixin,
    DocumentTypeMetadataTypeTestMixin
)
from .mixins.metadata_type_mixins import MetadataTypeAPIViewTestMixin


class MetadataTypeAPITestCase(MetadataTypeAPIViewTestMixin, BaseAPITestCase):
    def test_metadata_type_create_api_view_no_permission(self):
        self._clear_events()

        response = self._request_test_metadata_type_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
        self.assertEqual(MetadataType.objects.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_create_api_view_with_permission(self):
        self.grant_permission(permission=permission_metadata_type_create)

        self._clear_events()

        response = self._request_test_metadata_type_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        metadata_type = MetadataType.objects.first()
        self.assertEqual(response.data['id'], metadata_type.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_metadata_type)
        self.assertEqual(events[0].verb, event_metadata_type_created.id)

    def test_metadata_type_delete_api_view_no_permission(self):
        self._create_test_metadata_type()

        self._clear_events()

        response = self._request_test_metadata_type_delete_api_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(MetadataType.objects.count(), 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_delete_api_view_with_access(self):
        self._create_test_metadata_type()
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_delete
        )

        self._clear_events()

        response = self._request_test_metadata_type_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(MetadataType.objects.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_detail_api_view_no_permission(self):
        self._create_test_metadata_type()

        self._clear_events()

        response = self._request_test_metadata_type_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_detail_api_view_with_access(self):
        self._create_test_metadata_type()
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_view
        )

        self._clear_events()

        response = self._request_test_metadata_type_detail_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self.assertEqual(
            response.data['label'], self._test_metadata_type.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_patch_api_view_no_permission(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self._test_metadata_type
        )

        self._clear_events()

        response = self._request_test_metadata_type_edit_api_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_metadata_type.refresh_from_db()
        self.assertEqual(
            self._model_instance_to_dictionary(
                instance=self._test_metadata_type
            ), metadata_type_values
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_patch_api_view_with_access(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self._test_metadata_type
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_test_metadata_type_edit_api_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_metadata_type.refresh_from_db()
        self.assertNotEqual(
            self._model_instance_to_dictionary(
                instance=self._test_metadata_type
            ), metadata_type_values
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_metadata_type)
        self.assertEqual(events[0].verb, event_metadata_type_edited.id)

    def test_metadata_type_put_api_view_no_permission(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self._test_metadata_type
        )

        self._clear_events()

        response = self._request_test_metadata_type_edit_api_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self._test_metadata_type.refresh_from_db()
        self.assertEqual(
            self._model_instance_to_dictionary(
                instance=self._test_metadata_type
            ), metadata_type_values
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_put_api_view_with_access(self):
        self._create_test_metadata_type()
        metadata_type_values = self._model_instance_to_dictionary(
            instance=self._test_metadata_type
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_test_metadata_type_edit_api_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        self._test_metadata_type.refresh_from_db()
        self.assertNotEqual(
            self._model_instance_to_dictionary(
                instance=self._test_metadata_type
            ), metadata_type_values
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_metadata_type)
        self.assertEqual(events[0].verb, event_metadata_type_edited.id)

    def test_metadata_type_list_api_view_no_permission(self):
        self._create_test_metadata_type()

        self._clear_events()

        response = self._request_test_metadata_type_list_api_view()

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_metadata_type_list_api_view_with_access(self):
        self._create_test_metadata_type()
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_view
        )

        self._clear_events()

        response = self._request_test_metadata_type_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['label'],
            self._test_metadata_type.label
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentTypeMetadataTypeAPITestCase(
    DocumentTestMixin, DocumentTypeMetadataTypeAPIViewTestMixin,
    DocumentTypeMetadataTypeTestMixin, BaseAPITestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        self._create_test_metadata_type()

    def test_document_type_metadata_type_create_api_view_no_permission(self):
        self._clear_events()

        response = self._request_document_type_metadata_type_create_api_view()

        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
        self.assertEqual(self._test_document_type.metadata.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_create_api_view_with_document_type_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)

        self.assertEqual(self._test_document_type.metadata.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_create_api_view_with_metadata_type_access(self):
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(self._test_document_type.metadata.count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_create_api_view_with_full_access(self):
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_create_api_view()
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertEqual(response.data['id'], document_type_metadata_type.pk)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_metadata_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_type)
        self.assertEqual(
            events[0].verb, event_metadata_type_relationship_updated.id
        )

    def test_document_type_metadata_type_create_duplicate_api_view(self):
        self._create_test_document_type_metadata_type()
        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_create_api_view()

        self.assertEqual(response.status_code, status.HTTP_400_BAD_REQUEST)
        self.assertEqual(list(response.data.keys())[0], 'non_field_errors')

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_delete_api_view_no_permission(self):
        self._create_test_document_type_metadata_type()

        self._clear_events()

        response = self._request_document_type_metadata_type_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(self._test_document_type.metadata.count(), 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_delete_api_view_with_document_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(self._test_document_type.metadata.count(), 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_delete_api_view_with_metadata_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        self.assertEqual(self._test_document_type.metadata.count(), 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_delete_api_view_with_full_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_delete_api_view()
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        self.assertEqual(self._test_document_type.metadata.all().count(), 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_metadata_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_type)
        self.assertEqual(
            events[0].verb, event_metadata_type_relationship_updated.id
        )

    def test_document_type_metadata_type_list_api_view_no_permission(self):
        self._create_test_document_type_metadata_type()

        self._clear_events()

        response = self._request_document_type_metadata_type_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_list_api_view_document_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_view
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(response.data['count'], 0)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_list_api_view_metadata_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_view
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_list_api_view_with_full_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_view
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_view
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_list_api_view()
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(
            response.data['results'][0]['id'],
            self._test_document_type_metadata_type.pk
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_patch_api_view_no_permission(self):
        self._create_test_document_type_metadata_type()

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_patch_api_view_with_document_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_patch_api_view_with_metadata_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_patch_api_view_with_full_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_patch()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertEqual(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_metadata_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_type)
        self.assertEqual(
            events[0].verb, event_metadata_type_relationship_updated.id
        )

    def test_document_type_metadata_type_put_api_view_no_permission(self):
        self._create_test_document_type_metadata_type()

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_put_api_view_with_document_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_put_api_view_with_metadata_type_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertFalse(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_type_metadata_type_put_api_view_with_full_access(self):
        self._create_test_document_type_metadata_type()

        self.grant_access(
            obj=self._test_document_type,
            permission=permission_document_type_edit
        )
        self.grant_access(
            obj=self._test_metadata_type,
            permission=permission_metadata_type_edit
        )

        self._clear_events()

        response = self._request_document_type_metadata_type_edit_api_view_via_put()
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        document_type_metadata_type = DocumentTypeMetadataType.objects.first()
        self.assertEqual(document_type_metadata_type.required, True)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_metadata_type)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_type)
        self.assertEqual(
            events[0].verb, event_metadata_type_relationship_updated.id
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.document_states.classes import WorkflowAction

from .models.document_type_models import DocumentType


class DocumentTypeChangeAction(WorkflowAction):
    form_field_widgets = {
        'document_type': {
            'class': 'django.forms.widgets.Select', 'kwargs': {
                'attrs': {'class': 'select2'}
            }
        }
    }
    form_fields = {
        'document_type': {
            'label': _('Document type'),
            'class': 'django.forms.ModelChoiceField', 'kwargs': {
                'help_text': _(
                    'New document type for the workflow document.'
                ), 'queryset': DocumentType.objects.all(), 'required': True
            }
        }
    }
    label = _('Change document type')

    @classmethod
    def get_form_fieldsets(cls):
        fieldsets = super().get_form_fieldsets()

        fieldsets += (
            (
                _('Document type'), {
                    'fields': ('document_type',)
                }
            ),
        )
        return fieldsets

    def execute(self, context):
        context['workflow_instance'].document._document_type_change(
            document_type=self.get_document_type()
        )

    def get_document_type(self):
        return DocumentType.objects.get(
            pk=self.kwargs.get('document_type')
        )


class TrashDocumentAction(WorkflowAction):
    label = _('Send document to trash')

    def execute(self, context):
        context['workflow_instance'].document.delete()



from mayan.apps.appearance.classes import Icon

# App

icon_document_file_content = Icon(driver_name='fontawesome', symbol='font')

# Document file

icon_document_file_content_single_delete = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='times'
)
icon_document_file_content_multiple_delete = icon_document_file_content_single_delete
icon_document_file_content_detail = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_file_content_download = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-down'
)
icon_document_file_parsing_multiple_submit = Icon(
    driver_name='fontawesome-dual', primary_symbol='font',
    secondary_symbol='arrow-right'
)
icon_document_file_parsing_single_submit = icon_document_file_parsing_multiple_submit

# Document file page

icon_document_file_page_content_detail = icon_document_file_content

# Document type

icon_document_type_parsing_settings = Icon(
    driver_name='fontawesome', symbol='font'
)
icon_document_type_parsing_submit = icon_document_file_parsing_multiple_submit



from django.db import migrations


def code_source_task_path_update(apps, schema_editor):
    PeriodicTask = apps.get_model(
        app_label='django_celery_beat', model_name='PeriodicTask'
    )

    PeriodicTask.objects.using(
        alias=schema_editor.connection.alias
    ).filter(
        task='mayan.apps.sources.tasks.task_source_process_document'
    ).update(
        task='mayan.apps.sources.tasks.task_source_backend_action_execute'
    )


def reverse_code_source_task_path_update(apps, schema_editor):
    PeriodicTask = apps.get_model(
        app_label='django_celery_beat', model_name='PeriodicTask'
    )

    PeriodicTask.objects.using(
        alias=schema_editor.connection.alias
    ).filter(
        task='mayan.apps.sources.tasks.task_source_backend_action_execute'
    ).update(
        task='mayan.apps.sources.tasks.task_source_process_document'
    )


class Migration(migrations.Migration):
    dependencies = [
        ('sources', '0030_documentfilesourcemetadata'),
        ('django_celery_beat', '0016_alter_crontabschedule_timezone')
    ]

    operations = [
        migrations.RunPython(
            code=code_source_task_path_update,
            reverse_code=reverse_code_source_task_path_update
        )
    ]



from django.conf.urls import url

from .views import (
    CacheDetailView, CacheListView, CachePartitionDetailView,
    CachePartitionPurgeView, CachePurgeView
)

urlpatterns = [
    url(
        regex=r'^caches/$', name='cache_list', view=CacheListView.as_view()
    ),
    url(
        regex=r'^caches/(?P<cache_id>\d+)/detail/$', name='cache_detail',
        view=CacheDetailView.as_view()
    ),
    url(
        regex=r'^caches/(?P<cache_id>\d+)/purge/$', name='cache_purge',
        view=CachePurgeView.as_view()
    ),
    url(
        regex=r'^caches/multiple/purge/$', name='cache_multiple_purge',
        view=CachePurgeView.as_view()
    ),
    url(
        regex=r'^caches/(?P<cache_partition_id>\d+)/$',
        name='cache_partition_detail',
        view=CachePartitionDetailView.as_view()
    ),
    url(
        regex=r'^apps/(?P<app_label>[-\w]+)/models/(?P<model_name>[-\w]+)/objects/(?P<object_id>\d+)/cache_partitions/purge/$',
        name='cache_partitions_purge', view=CachePartitionPurgeView.as_view()
    )
]



from rest_framework.exceptions import ParseError

from mayan.apps.rest_api import generics

from .api_view_mixins import SearchModelAPIViewMixin
from .exceptions import DynamicSearchException
from .search_models import SearchModel
from .serializers import (
    DummySearchResultModelSerializer, SearchModelSerializer
)
from .view_mixins import SearchResultViewMixin


class APISearchView(
    SearchResultViewMixin, SearchModelAPIViewMixin, generics.ListAPIView
):
    """
    get: Perform a search operation.
    """

    def get_serializer_class(self):
        if getattr(self, 'swagger_fake_view', False):
            return DummySearchResultModelSerializer
        else:
            return self.search_model.serializer

    def get_source_queryset(self):
        try:
            return self.get_search_queryset()
        except DynamicSearchException as exception:
            raise ParseError(
                detail=str(exception)
            )


class APISearchModelList(generics.ListAPIView):
    """
    get: Returns a list of all the available search models.
    """

    serializer_class = SearchModelSerializer

    def get_source_queryset(self):
        # This changes after the initial startup as search models are
        # automatically loaded.
        return SearchModel.all()



from django.apps import apps
from django.test import TestCase, TransactionTestCase, tag

from django_test_migrations.contrib.unittest_case import MigratorTestCase

from mayan.apps.acls.tests.mixins import ACLTestCaseMixin
from mayan.apps.converter.tests.mixins import LayerTestCaseMixin
from mayan.apps.events.tests.mixins import EventTestCaseMixin
from mayan.apps.permissions.tests.mixins import PermissionTestCaseMixin
from mayan.apps.smart_settings.tests.mixins import SmartSettingsTestCaseMixin
from mayan.apps.user_management.tests.mixins import UserTestMixin

from ..literals import EXCLUDE_TEST_TAG

from .mixins import (
    ClientMethodsTestCaseMixin, ConnectionsCheckTestCaseMixin,
    ContentTypeCheckTestCaseMixin, DelayTestCaseMixin,
    DescriptorLeakCheckTestCaseMixin, DownloadTestCaseMixin,
    ModelTestCaseMixin, OpenFileCheckTestCaseMixin,
    RandomPrimaryKeyModelMonkeyPatchMixin, SilenceLoggerTestCaseMixin,
    TempfileCheckTestCasekMixin, TestModelTestCaseMixin,
    TestViewTestCaseMixin
)


class BaseTestCaseMixin(
    DelayTestCaseMixin, LayerTestCaseMixin, SilenceLoggerTestCaseMixin,
    ConnectionsCheckTestCaseMixin, DownloadTestCaseMixin, EventTestCaseMixin,
    RandomPrimaryKeyModelMonkeyPatchMixin, ACLTestCaseMixin,
    ModelTestCaseMixin, OpenFileCheckTestCaseMixin,
    DescriptorLeakCheckTestCaseMixin, PermissionTestCaseMixin,
    SmartSettingsTestCaseMixin, TempfileCheckTestCasekMixin, UserTestMixin,
    TestModelTestCaseMixin
):
    """
    This is the most basic test case mixin class any test in the project
    should use.
    TestModelTestCaseMixin must go before TestViewTestCaseMixin to allow
    the test object to be available when the test view is prepared.

    Favor `OpenFileCheckTestCaseMixin` over
    `DescriptorLeakCheckTestCaseMixin` as it provides more context.
    """
    _skip_file_descriptor_test = True
    _skip_open_file_leak_test = True


class BaseTestCase(BaseTestCaseMixin, TestCase):
    """
    All the project test mixin on top of Django test case class.
    """


class BaseTransactionTestCase(BaseTestCaseMixin, TransactionTestCase):
    """
    All the project test mixin on top of Django transaction test case class.
    """


class GenericViewTestCase(
    ClientMethodsTestCaseMixin, ContentTypeCheckTestCaseMixin,
    TestViewTestCaseMixin, BaseTestCase
):
    """
    A generic view test case built on top of the base test case providing
    a single, user customizable view to test object resolution and shorthand
    HTTP method functions.
    """


class GenericTransactionViewTestCase(
    ClientMethodsTestCaseMixin, ContentTypeCheckTestCaseMixin,
    TestViewTestCaseMixin, BaseTransactionTestCase
):
    """
    A generic view test case built on top of the transaction base test case
    providing a single, user customizable view to test object resolution
    and shorthand HTTP method functions.
    """


@tag(EXCLUDE_TEST_TAG)
class MayanMigratorTestCase(MigratorTestCase):
    def tearDown(self):
        ContentType = apps.get_model(
            app_label='contenttypes', model_name='ContentType'
        )

        ContentType.objects.clear_cache()

        super().tearDown()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.dynamic_search.search_models import SearchModel

from .permissions import permission_role_view

role_search = SearchModel(
    app_label='permissions', model_name='Role',
    permission=permission_role_view,
    serializer_path='mayan.apps.permissions.serializers.RoleSerializer'
)

role_search.add_model_field(
    field='label', label=_('Label')
)

role_search.add_model_field(
    field='groups__name', label=_('Group name')
)



from mayan.apps.testing.tests.base import GenericViewTestCase

from .mixins import AnnouncementTestMixin, AnnouncementViewTestMixin

from ..events import event_announcement_created, event_announcement_edited
from ..models import Announcement
from ..permissions import (
    permission_announcement_create, permission_announcement_delete,
    permission_announcement_edit, permission_announcement_view,
)


class AnnouncementViewTestCase(
    AnnouncementTestMixin, AnnouncementViewTestMixin, GenericViewTestCase
):
    def test_announcement_create_view_no_permission(self):
        announcement_count = Announcement.objects.count()

        self._clear_events()

        response = self._request_test_announcement_create_view()
        self.assertEqual(response.status_code, 403)

        self.assertEqual(Announcement.objects.count(), announcement_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_announcement_create_view_with_permissions(self):
        self.grant_permission(permission=permission_announcement_create)

        announcement_count = Announcement.objects.count()

        self._clear_events()

        response = self._request_test_announcement_create_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Announcement.objects.count(), announcement_count + 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_announcement)
        self.assertEqual(events[0].verb, event_announcement_created.id)

    def test_announcement_delete_view_no_permission(self):
        self._create_test_announcement()

        announcement_count = Announcement.objects.count()

        self._clear_events()

        response = self._request_test_announcement_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(Announcement.objects.count(), announcement_count)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_announcement_delete_view_with_access(self):
        self._create_test_announcement()

        self.grant_access(
            obj=self._test_announcement, permission=permission_announcement_delete
        )

        announcement_count = Announcement.objects.count()

        self._clear_events()

        response = self._request_test_announcement_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(Announcement.objects.count(), announcement_count - 1)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_announcement_edit_view_no_permission(self):
        self._create_test_announcement()

        announcement_label = self._test_announcement.label

        self._clear_events()

        response = self._request_test_announcement_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_announcement.refresh_from_db()
        self.assertEqual(self._test_announcement.label, announcement_label)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_announcement_edit_view_with_access(self):
        self._create_test_announcement()

        self.grant_access(
            obj=self._test_announcement, permission=permission_announcement_edit
        )

        announcement_label = self._test_announcement.label

        self._clear_events()

        response = self._request_test_announcement_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_announcement.refresh_from_db()
        self.assertNotEqual(self._test_announcement.label, announcement_label)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_announcement)
        self.assertEqual(events[0].verb, event_announcement_edited.id)

    def test_announcement_list_view_with_no_permission(self):
        self._create_test_announcement()

        self._clear_events()

        response = self._request_test_announcement_list_view()
        self.assertNotContains(
            response=response, text=self._test_announcement.label, status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_announcement_list_view_with_access(self):
        self._create_test_announcement()

        self.grant_access(
            obj=self._test_announcement, permission=permission_announcement_view
        )

        self._clear_events()

        response = self._request_test_announcement_list_view()
        self.assertContains(
            response=response, text=self._test_announcement.label, status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from mayan.apps.appearance.classes import Icon
from mayan.apps.documents.icons import icon_document_list

icon_menu_tags = Icon(driver_name='fontawesome', symbol='tags')

# Documents

icon_document_multiple_tag_multiple_attach = Icon(
    driver_name='fontawesome-dual', primary_symbol='tag',
    secondary_symbol='arrow-right'
)
icon_document_multiple_tag_multiple_remove = Icon(
    driver_name='fontawesome-dual', primary_symbol='tag',
    secondary_symbol='arrow-left'
)
icon_document_tag_multiple_attach = Icon(
    driver_name='fontawesome-dual', primary_symbol='tag',
    secondary_symbol='arrow-right'
)
icon_document_tag_multiple_remove = Icon(
    driver_name='fontawesome-dual', primary_symbol='tag',
    secondary_symbol='arrow-left'
)
icon_document_tag_list = Icon(driver_name='fontawesome', symbol='tags')

# Tags

icon_tag_create = Icon(
    driver_name='fontawesome-dual', primary_symbol='tag',
    secondary_symbol='plus'
)
icon_tag_single_delete = Icon(driver_name='fontawesome', symbol='times')
icon_tag_multiple_delete = Icon(driver_name='fontawesome', symbol='times')
icon_tag_document_list = icon_document_list
icon_tag_edit = Icon(driver_name='fontawesome', symbol='pencil-alt')
icon_tag_list = Icon(driver_name='fontawesome', symbol='tag')



from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.search import search_model_document
from mayan.apps.dynamic_search.search_models import SearchModel

from .permissions import permission_signature_capture_view


#  Document

search_model_document.add_model_field(
    field='signature_captures__text', label=_('Signature capture text')
)
search_model_document.add_model_field(
    field='signature_captures__user__first_name',
    label=_('Signature capture user first name')
)
search_model_document.add_model_field(
    field='signature_captures__user__last_name',
    label=_('Signature capture user last name')
)
search_model_document.add_model_field(
    field='signature_captures__user__username',
    label=_('Signature capture user username')
)

# Signature capture

search_model_signature_capture = SearchModel(
    app_label='signature_captures', model_name='SignatureCapture',
    permission=permission_signature_capture_view,
    serializer_path='mayan.apps.signature_captures.serializers.SignatureCaptureSerializer'
)
search_model_signature_capture.add_model_field(
    field='document__document_type__label', label=_('Document type label')
)
search_model_signature_capture.add_model_field(
    field='document__label', label=_('Document label')
)
search_model_signature_capture.add_model_field(field='text')
search_model_signature_capture.add_model_field(
    field='user__first_name', label=_('User first name')
)
search_model_signature_capture.add_model_field(
    field='user__last_name', label=_('User last name')
)
search_model_signature_capture.add_model_field(
    field='user__username', label=_('User username')
)



from django.utils.translation import ugettext_lazy as _

from mayan.apps.dependencies.classes import JavaScriptDependency

JavaScriptDependency(
    label=_('JavaScript image cropper'), module=__name__, name='cropperjs',
    version_string='=1.5.12'
)
JavaScriptDependency(
    module=__name__, name='jquery-cropper', version_string='=1.0.1'
)



from django.db import models
from django.utils.translation import ugettext_lazy as _

from mayan.apps.documents.models.document_file_models import DocumentFile
from mayan.apps.documents.models.document_type_models import DocumentType

from .managers import DocumentTypeSettingsManager
from .model_mixins import (
    DocumentFileDriverEntryBusinessLogicMixin, StoredDriverBusinessLogicMixin
)


class DocumentFileDriverEntry(
    DocumentFileDriverEntryBusinessLogicMixin, models.Model
):
    driver = models.ForeignKey(
        on_delete=models.CASCADE, related_name='driver_entries',
        to='StoredDriver', verbose_name=_('Driver')
    )
    document_file = models.ForeignKey(
        on_delete=models.CASCADE, related_name='file_metadata_drivers',
        to=DocumentFile, verbose_name=_('Document file')
    )

    class Meta:
        ordering = ('document_file', 'driver')
        unique_together = ('driver', 'document_file')
        verbose_name = _('Document file driver entry')
        verbose_name_plural = _('Document file driver entries')

    def __str__(self):
        return str(self.driver)


class DocumentTypeSettings(models.Model):
    """
    Model to store the file metadata settings for a document type.
    """
    document_type = models.OneToOneField(
        on_delete=models.CASCADE, related_name='file_metadata_settings',
        to=DocumentType, unique=True, verbose_name=_('Document type')
    )
    auto_process = models.BooleanField(
        default=True, help_text=_(
            'Automatically queue newly created documents for processing.'
        ), verbose_name=_('Auto process')
    )

    objects = DocumentTypeSettingsManager()

    class Meta:
        verbose_name = _('Document type settings')
        verbose_name_plural = _('Document types settings')

    def natural_key(self):
        return self.document_type.natural_key()
    natural_key.dependencies = ['documents.DocumentType']


class FileMetadataEntry(models.Model):
    document_file_driver_entry = models.ForeignKey(
        on_delete=models.CASCADE, related_name='entries',
        to=DocumentFileDriverEntry,
        verbose_name=_('Document file driver entry')
    )
    key = models.CharField(
        db_index=True, help_text=_('Name of the file metadata entry.'),
        max_length=255, verbose_name=_('Key')
    )
    value = models.TextField(
        blank=True, help_text=_('Value of the file metadata entry.'),
        max_length=255, verbose_name=_('Value')
    )

    class Meta:
        ordering = ('key', 'value')
        verbose_name = _('File metadata entry')
        verbose_name_plural = _('File metadata entries')

    def __str__(self):
        return '{}: {}: {}'.format(
            self.document_file_driver_entry, self.key, self.value
        )


class StoredDriver(StoredDriverBusinessLogicMixin, models.Model):
    driver_path = models.CharField(
        max_length=255, unique=True, verbose_name=_('Driver path')
    )
    internal_name = models.CharField(
        db_index=True, max_length=128, unique=True,
        verbose_name=_('Internal name')
    )

    class Meta:
        ordering = ('internal_name',)
        verbose_name = _('Driver')
        verbose_name_plural = _('Drivers')

    def __str__(self):
        return str(self.driver_label)



from django.db.models import Q

from mayan.apps.converter.layers import layer_saved_transformations

from ...literals import DEFAULT_DOCUMENT_FILE_ACTION_NAME, PAGE_RANGE_ALL
from ...models.document_file_models import DocumentFile

from ..literals import (
    TEST_DOCUMENT_FILE_COMMENT, TEST_DOCUMENT_FILE_COMMENT_EDITED,
    TEST_DOCUMENT_FILE_FILENAME_EDITED, TEST_FILE_SMALL_PATH,
    TEST_TRANSFORMATION_ARGUMENT, TEST_TRANSFORMATION_CLASS
)


class DocumentFileAPIViewTestMixin:
    def _request_test_document_file_delete_api_view(self):
        return self.delete(
            viewname='rest_api:documentfile-detail', kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document.file_latest.pk
            }
        )

    def _request_test_document_file_detail_api_view(self):
        return self.get(
            viewname='rest_api:documentfile-detail', kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document.file_latest.pk
            }
        )

    def _request_test_document_file_list_api_view(self):
        return self.get(
            viewname='rest_api:documentfile-list', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_test_document_file_upload_api_view(self):
        pk_list = list(
            DocumentFile.objects.values_list('pk', flat=True)
        )

        with open(file=TEST_FILE_SMALL_PATH, mode='rb') as file_descriptor:
            response = self.post(
                viewname='rest_api:documentfile-list', kwargs={
                    'document_id': self._test_document.pk,
                }, data={
                    'action_name': DEFAULT_DOCUMENT_FILE_ACTION_NAME,
                    'comment': '', 'file_new': file_descriptor
                }
            )

        try:
            self._test_document_file = DocumentFile.objects.get(
                ~Q(pk__in=pk_list)
            )
            self._test_document.refresh_from_db()
            self._test_document_version = self._test_document.versions.last()
            self._test_document_version_page_list = self._test_document_version.pages.all()
            self._test_document_version_page = self._test_document_version_page_list.first()
        except DocumentFile.DoesNotExist:
            self._test_document_file = None

        return response


class DocumentFileLinkTestMixin:
    def _resolve_test_document_file_link(self, test_link):
        self.add_test_view(test_object=self._test_document_file)
        context = self.get_test_view()
        return test_link.resolve(context=context)


class DocumentFileTestMixin:
    def _upload_test_document_file(self, action_name=None, user=None):
        self._calculate_test_document_file_path()

        if not action_name:
            action_name = DEFAULT_DOCUMENT_FILE_ACTION_NAME

        document_file_count = DocumentFile.objects.count()

        pk_list = list(
            DocumentFile.objects.values_list('pk', flat=True)
        )

        with open(file=self._test_document_path, mode='rb') as file_object:
            self._test_document.files_upload(
                action_name=action_name, comment=TEST_DOCUMENT_FILE_COMMENT,
                file_object=file_object,
                filename='test_document_file_{}'.format(document_file_count),
                user=user
            )

        self._test_document_file = DocumentFile.objects.get(
            ~Q(pk__in=pk_list)
        )
        self._test_document_file_page = self._test_document_file.pages.first()
        self._test_document_file_page_list.extend(
            list(
                self._test_document_file.pages.all()
            )
        )
        self._test_document_file_list.append(self._test_document_file)

        for _test_document_version_list in self._test_document_version_list:
            _test_document_version_list.refresh_from_db()

        self._test_document_version = self._test_document.versions.last()
        self._test_document_version_list.append(self._test_document_version)

        self._test_document_version_page = self._test_document_version.pages.first()
        self._test_document_version_page_list.extend(
            list(
                self._test_document_version.pages.all()
            )
        )


class DocumentFileViewTestMixin:
    def _request_test_document_file_delete_view(self, document_file):
        return self.post(
            viewname='documents:document_file_delete', kwargs={
                'document_file_id': document_file.pk
            }
        )

    def _request_test_document_file_multiple_delete_view(self):
        return self.post(
            viewname='documents:document_file_multiple_delete', data={
                'id_list': self._test_document_file.pk
            }
        )

    def _request_test_document_file_edit_view(self):
        return self.post(
            viewname='documents:document_file_edit', kwargs={
                'document_file_id': self._test_document_file.pk
            }, data={
                'comment': TEST_DOCUMENT_FILE_COMMENT_EDITED,
                'filename': TEST_DOCUMENT_FILE_FILENAME_EDITED
            }
        )

    def _request_test_document_file_introspect_multiple_view(self):
        return self.post(
            viewname='documents:document_file_introspect_multiple',
            data={'id_list': self._test_document_file.pk}
        )

    def _request_test_document_file_introspect_single_view(self):
        return self.post(
            viewname='documents:document_file_introspect_single',
            kwargs={'document_file_id': self._test_document_file.pk}
        )

    def _request_test_document_file_list_view(self):
        return self.get(
            viewname='documents:document_file_list', kwargs={
                'document_id': self._test_document.pk
            }
        )

    def _request_test_document_file_preview_view(self):
        return self.get(
            viewname='documents:document_file_preview', kwargs={
                'document_file_id': self._test_document_file.pk
            }
        )

    def _request_test_document_file_print_form_view(self):
        return self.get(
            viewname='documents:document_file_print_form', kwargs={
                'document_file_id': self._test_document_file.pk,
            }, data={
                'page_group': PAGE_RANGE_ALL
            }
        )

    def _request_test_document_file_print_view(self):
        return self.get(
            viewname='documents:document_file_print_view', kwargs={
                'document_file_id': self._test_document_file.pk,
            }, query={
                'page_group': PAGE_RANGE_ALL
            }
        )

    def _request_test_document_file_properties_view(self):
        return self.get(
            viewname='documents:document_file_properties', kwargs={
                'document_file_id': self._test_document_file.pk
            }
        )


class DocumentFilePageAPIViewTestMixin:
    def _request_test_document_file_page_detail_api_view(self):
        return self.get(
            viewname='rest_api:documentfilepage-detail', kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk,
                'document_file_page_id': self._test_document_file_page.pk
            }
        )

    def _request_test_document_file_page_image_api_view(
        self, maximum_layer_order=None
    ):
        return self.get(
            viewname='rest_api:documentfilepage-image', kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk,
                'document_file_page_id': self._test_document_file_page.pk
            }, query={'maximum_layer_order': maximum_layer_order}
        )

    def _request_test_document_file_page_list_api_view(self):
        return self.get(
            viewname='rest_api:documentfilepage-list', kwargs={
                'document_id': self._test_document.pk,
                'document_file_id': self._test_document_file.pk,
            }
        )


class DocumentFilePageViewTestMixin:
    def _request_test_document_file_page_list_view(self):
        return self.get(
            viewname='documents:document_file_page_list', kwargs={
                'document_file_id': self._test_document_file.pk
            }
        )

    def _request_test_document_file_page_rotate_left_view(self):
        return self.post(
            viewname='documents:document_file_page_rotate_left', kwargs={
                'document_file_page_id': self._test_document_file_page.pk
            }
        )

    def _request_test_document_file_page_rotate_right_view(self):
        return self.post(
            viewname='documents:document_file_page_rotate_right', kwargs={
                'document_file_page_id': self._test_document_file_page.pk
            }
        )

    def _request_test_document_file_page_view(self, document_file_page):
        return self.get(
            viewname='documents:document_file_page_view', kwargs={
                'document_file_page_id': document_file_page.pk,
            }
        )

    def _request_test_document_file_page_zoom_in_view(self):
        return self.post(
            viewname='documents:document_file_page_zoom_in', kwargs={
                'document_file_page_id': self._test_document_file_page.pk
            }
        )

    def _request_test_document_file_page_zoom_out_view(self):
        return self.post(
            viewname='documents:document_file_page_zoom_out', kwargs={
                'document_file_page_id': self._test_document_file_page.pk
            }
        )


class DocumentFileTransformationTestMixin:
    def _create_document_file_transformation(self):
        layer_saved_transformations.add_transformation_to(
            obj=self._test_document_file.pages.first(),
            transformation_class=TEST_TRANSFORMATION_CLASS,
            arguments=TEST_TRANSFORMATION_ARGUMENT
        )


class DocumentFileTransformationViewTestMixin:
    def _request_test_document_file_transformations_clear_view(self):
        return self.post(
            viewname='documents:document_file_transformations_clear',
            kwargs={'document_file_id': self._test_document_file.pk}
        )

    def _request_test_document_file_multiple_transformations_clear_view(self):
        return self.post(
            viewname='documents:document_file_multiple_transformations_clear',
            data={'id_list': self._test_document_file.pk}
        )

    def _request_test_document_file_transformations_clone_view(self):
        return self.post(
            viewname='documents:document_file_transformations_clone',
            kwargs={'document_file_id': self._test_document_file.pk}, data={
                'page': self._test_document_file.pages.first().pk
            }
        )



from django.utils.translation import ugettext_lazy as _

from mayan.apps.smart_settings.classes import SettingNamespace

from .literals import (
    DEFAULT_AUTHENTICATION_BACKEND, DEFAULT_AUTHENTICATION_BACKEND_ARGUMENTS,
    DEFAULT_AUTHENTICATION_DISABLE_PASSWORD_RESET
)

namespace = SettingNamespace(
    label=_('Authentication'), name='authentication'
)

setting_disable_password_reset = namespace.add_setting(
    default=DEFAULT_AUTHENTICATION_DISABLE_PASSWORD_RESET,
    global_name='AUTHENTICATION_DISABLE_PASSWORD_RESET', help_text=_(
        'Remove the "Forgot your password?" link on the login form used to '
        'trigger the password reset.'
    )
)
setting_authentication_backend = namespace.add_setting(
    default=DEFAULT_AUTHENTICATION_BACKEND,
    global_name='AUTHENTICATION_BACKEND',
    help_text=_(
        'Dotted path to the backend used to process user authentication.'
    )
)
setting_authentication_backend_arguments = namespace.add_setting(
    default=DEFAULT_AUTHENTICATION_BACKEND_ARGUMENTS,
    global_name='AUTHENTICATION_BACKEND_ARGUMENTS',
    help_text=_('Arguments for the AUTHENTICATION_BACKEND.')
)



from django.contrib import admin

from .models import StoredCredential


@admin.register(StoredCredential)
class StoredCredentialAdmin(admin.ModelAdmin):
    list_display = ('label', 'internal_name')



from mayan.apps.file_caching.events import event_cache_partition_purged
from mayan.apps.file_caching.models import CachePartitionFile
from mayan.apps.file_caching.permissions import permission_cache_partition_purge
from mayan.apps.file_caching.tests.mixins import CachePartitionViewTestMixin

from ..document_file_actions import (
    DocumentFileActionAppendNewPages, DocumentFileActionNothing
)
from ..events import (
    event_document_version_deleted, event_document_version_edited,
    event_document_version_page_created, event_document_version_page_deleted,
    event_document_viewed
)
from ..permissions import (
    permission_document_version_delete, permission_document_version_edit,
    permission_document_version_print, permission_document_version_view
)

from .base import GenericDocumentViewTestCase
from .mixins.document_file_mixins import DocumentFileTestMixin
from .mixins.document_version_mixins import (
    DocumentVersionModificationViewTestMixin, DocumentVersionTestMixin,
    DocumentVersionViewTestMixin
)


class DocumentVersionViewTestCase(
    DocumentVersionTestMixin, DocumentVersionViewTestMixin,
    GenericDocumentViewTestCase
):
    def test_document_version_active_view_no_permission(self):
        self._create_test_document_version()

        self._test_document.versions.first().active_set()

        self._clear_events()

        response = self._request_test_document_version_active_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()
        self.assertFalse(self._test_document_version.active)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_active_view_with_access(self):
        self._create_test_document_version()

        self._test_document.versions.first().active_set()

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._clear_events()

        response = self._request_test_document_version_active_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_version.refresh_from_db()
        self.assertTrue(self._test_document_version.active)

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(events[0].verb, event_document_version_edited.id)

    def test_trashed_document_version_active_view_with_access(self):
        self._create_test_document_version()

        self._test_document.versions.first().active_set()

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_version_active_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()
        self.assertFalse(self._test_document_version.active)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_single_delete_view_no_permission(self):
        self._create_test_document_version()

        document_version_count = self._test_document.versions.count()

        self._clear_events()

        response = self._request_test_document_version_single_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_single_delete_view_with_access(self):
        self._create_test_document_version()
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_delete
        )

        document_version_count = self._test_document.versions.count()

        self._clear_events()

        response = self._request_test_document_version_single_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_version_deleted.id)

    def test_document_version_multiple_delete_view_no_permission(self):
        self._create_test_document_version()

        document_version_count = self._test_document.versions.count()

        self._clear_events()

        response = self._request_test_document_version_multiple_delete_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_multiple_delete_view_with_access(self):
        self._create_test_document_version()

        document_version_count = self._test_document.versions.count()

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_delete
        )

        self._clear_events()

        response = self._request_test_document_version_multiple_delete_view()
        self.assertEqual(response.status_code, 302)

        self.assertEqual(
            self._test_document.versions.count(), document_version_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_version_deleted.id)

    def test_document_version_edit_view_no_permission(self):
        document_version_comment = self._test_document_version.comment

        self._clear_events()

        response = self._request_test_document_version_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()
        self.assertEqual(
            self._test_document_version.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_edit_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        document_version_comment = self._test_document_version.comment

        self._clear_events()

        response = self._request_test_document_version_edit_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_version.refresh_from_db()
        self.assertNotEqual(
            self._test_document_version.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(events[0].verb, event_document_version_edited.id)

    def test_trashed_document_version_edit_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        document_version_comment = self._test_document_version.comment

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_version_edit_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()
        self.assertEqual(
            self._test_document_version.comment,
            document_version_comment
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_list_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_list_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_view
        )

        self._clear_events()

        response = self._request_test_document_version_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=str(self._test_document_version)
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_list_view_with_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_document_version_view
        )

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_version_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_preview_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_preview_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_preview_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_view
        )

        self._clear_events()

        response = self._request_test_document_version_preview_view()
        self.assertContains(
            response=response, status_code=200,
            text=str(self._test_document_version)
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 1)

        self.assertEqual(events[0].action_object, self._test_document_version)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document)
        self.assertEqual(events[0].verb, event_document_viewed.id)

    def test_trashed_document_version_preview_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_view
        )

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_version_preview_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_print_form_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_print_form_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_print_form_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_print
        )

        self._clear_events()

        response = self._request_test_document_version_print_form_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_print_form_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_print
        )

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_version_print_form_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_print_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_version_print_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_print_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_print
        )

        self._clear_events()

        response = self._request_test_document_version_print_view()
        self.assertEqual(response.status_code, 200)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_version_print_view_with_access(self):
        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_print
        )

        self._test_document.delete()
        self._clear_events()

        response = self._request_test_document_version_print_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class DocumentVersionCachePurgeViewTestCase(
    CachePartitionViewTestMixin, GenericDocumentViewTestCase
):
    def test_document_version_cache_purge_no_permission(self):
        self._test_object = self._test_document_version
        self._inject_test_object_content_type()

        self._test_document_version.version_pages.first().generate_image()

        test_document_version_cache_partitions = self._test_document_version.get_cache_partitions()

        cache_partition_version_count = CachePartitionFile.objects.filter(
            partition__in=test_document_version_cache_partitions
        ).count()

        self._clear_events()

        response = self._request_test_object_file_cache_partition_purge_view()
        self.assertEqual(response.status_code, 404)

        self.assertEqual(
            CachePartitionFile.objects.filter(
                partition__in=test_document_version_cache_partitions
            ).count(), cache_partition_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_cache_purge_with_access(self):
        self._test_object = self._test_document_version
        self._inject_test_object_content_type()

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_cache_partition_purge
        )

        self._test_document_version.version_pages.first().generate_image()

        test_document_version_cache_partitions = self._test_document_version.get_cache_partitions()

        cache_partition_version_count = CachePartitionFile.objects.filter(
            partition__in=test_document_version_cache_partitions
        ).count()

        cache_partitions = self._test_document_version.get_cache_partitions()

        self._clear_events()

        response = self._request_test_object_file_cache_partition_purge_view()
        self.assertEqual(response.status_code, 302)

        self.assertNotEqual(
            CachePartitionFile.objects.filter(
                partition__in=test_document_version_cache_partitions
            ).count(), cache_partition_version_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 2)

        self.assertEqual(events[0].action_object, self._test_document_version)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, cache_partitions[0])
        self.assertEqual(events[0].verb, event_cache_partition_purged.id)

        self.assertEqual(events[1].action_object, self._test_document_version)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, cache_partitions[1])
        self.assertEqual(events[1].verb, event_cache_partition_purged.id)


class DocumentVersionModificationViewTestCase(
    DocumentFileTestMixin, DocumentVersionModificationViewTestMixin,
    DocumentVersionTestMixin, GenericDocumentViewTestCase
):
    def test_document_version_action_page_append_view_no_permission(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionNothing.backend_id
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_append_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_action_page_append_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionNothing.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_append_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count() + self._test_document_file_list[1].pages.count()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(events[1].action_object, self._test_document_version)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(
            events[1].target, self._test_document_version.pages[0]
        )
        self.assertEqual(
            events[1].verb, event_document_version_page_created.id
        )
        self.assertEqual(events[2].action_object, self._test_document_version)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(
            events[2].target, self._test_document_version.pages[1]
        )
        self.assertEqual(
            events[2].verb, event_document_version_page_created.id
        )

    def test_trashed_document_version_action_page_append_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionNothing.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_action_page_append_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count()
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_action_page_reset_view_no_permission(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionAppendNewPages.backend_id
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_reset_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count() + self._test_document_file_list[1].pages.count()
        )

        self.assertEqual(
            self._test_document_version.pages.all()[0].content_object,
            self._test_document_file_page_list[0]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_version_action_page_reset_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionAppendNewPages.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._clear_events()

        response = self._request_test_document_version_action_page_reset_view()
        self.assertEqual(response.status_code, 302)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count()
        )

        self.assertEqual(
            self._test_document_version.pages.all()[0].content_object,
            self._test_document_file_page_list[1]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        self.assertEqual(events[0].action_object, None)
        self.assertEqual(events[0].actor, self._test_case_user)
        self.assertEqual(events[0].target, self._test_document_version)
        self.assertEqual(
            events[0].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(events[1].action_object, None)
        self.assertEqual(events[1].actor, self._test_case_user)
        self.assertEqual(events[1].target, self._test_document_version)
        self.assertEqual(
            events[1].verb, event_document_version_page_deleted.id
        )

        self.assertEqual(events[2].action_object, self._test_document_version)
        self.assertEqual(events[2].actor, self._test_case_user)
        self.assertEqual(
            events[2].target, self._test_document_version.pages[0]
        )
        self.assertEqual(
            events[2].verb, event_document_version_page_created.id
        )

    def test_trashed_document_version_action_page_reset_view_with_access(self):
        self._upload_test_document_file(
            action_name=DocumentFileActionAppendNewPages.backend_id
        )

        self.grant_access(
            obj=self._test_document_version,
            permission=permission_document_version_edit
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_version_action_page_reset_view()
        self.assertEqual(response.status_code, 404)

        self._test_document_version.refresh_from_db()

        self.assertEqual(
            self._test_document_version.pages.count(),
            self._test_document_file_list[0].pages.count() + self._test_document_file_list[1].pages.count()
        )

        self.assertEqual(
            self._test_document_version.pages.all()[0].content_object,
            self._test_document_file_page_list[0]
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from mayan.apps.documents.permissions import permission_document_view
from mayan.apps.documents.tests.base import GenericDocumentViewTestCase

from ..permissions import permission_index_instance_view

from .literals import TEST_INDEX_TEMPLATE_LABEL
from .mixins import (
    DocumentIndexInstanceViewTestMixin, IndexInstanceTestMixin,
    IndexInstanceViewTestMixin, IndexTemplateTestMixin
)


class DocumentIndexInstanceViewTestCase(
    IndexInstanceTestMixin, DocumentIndexInstanceViewTestMixin,
    IndexTemplateTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        # Create test document after the index template is created.
        self._create_test_document_stub()
        self._populate_test_index_instance_node()

    def test_document_index_instance_list_view_no_permission(self):
        self._clear_events()

        response = self._request_test_document_index_instance_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_index_instance_list_view_with_index_access(self):
        self.grant_access(
            obj=self._test_index_template,
            permission=permission_index_instance_view
        )

        self._clear_events()

        response = self._request_test_document_index_instance_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_index_instance_list_view_with_document_access(self):
        self.grant_access(
            obj=self._test_document,
            permission=permission_index_instance_view
        )

        self._clear_events()

        response = self._request_test_document_index_instance_list_view()
        self.assertNotContains(
            response=response, status_code=200,
            text=TEST_INDEX_TEMPLATE_LABEL
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_document.pk
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_document_index_instance_list_view_with_full_access(self):
        self.grant_access(
            obj=self._test_index_template,
            permission=permission_index_instance_view
        )
        self.grant_access(
            obj=self._test_document,
            permission=permission_index_instance_view
        )

        self._clear_events()

        response = self._request_test_document_index_instance_list_view()
        self.assertContains(
            response=response, status_code=200,
            text=TEST_INDEX_TEMPLATE_LABEL
        )
        self.assertContains(
            response=response, status_code=200,
            text=self._test_document.pk
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_trashed_document_index_list_view_with_full_access(self):
        self.grant_access(
            obj=self._test_index_template,
            permission=permission_index_instance_view
        )
        self.grant_access(
            obj=self._test_document,
            permission=permission_index_instance_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_document_index_instance_list_view()
        self.assertEqual(response.status_code, 404)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)


class IndexInstanceViewTestCase(
    IndexInstanceTestMixin, IndexInstanceViewTestMixin,
    IndexTemplateTestMixin, GenericDocumentViewTestCase
):
    auto_upload_test_document = False

    def setUp(self):
        super().setUp()
        # Create test document after the index template is created.
        self._create_test_document_stub()
        self._populate_test_index_instance_node()

    def test_index_instance_root_node_view_no_permission(self):
        self._clear_events()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self._test_index_instance_root_node
        )
        self.assertEqual(response.status_code, 403)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_index_instance_root_node_view_with_access(self):
        self.grant_access(
            obj=self._test_index_template,
            permission=permission_index_instance_view
        )

        self._clear_events()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self._test_index_instance_root_node
        )
        self.assertContains(
            response=response, text=TEST_INDEX_TEMPLATE_LABEL,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_index_instance_document_node_view_no_permission(self):
        self._clear_events()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self._test_index_instance_node
        )
        self.assertEqual(response.status_code, 403)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_index_instance_document_node_view_with_index_access(self):
        self.grant_access(
            obj=self._test_index_template,
            permission=permission_index_instance_view
        )

        self._clear_events()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self._test_index_instance_node
        )
        self.assertContains(
            response=response, text=TEST_INDEX_TEMPLATE_LABEL,
            status_code=200
        )
        self.assertNotContains(
            response=response, text=self._test_document.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_index_instance_document_node_view_with_document_access(self):
        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self._test_index_instance_node
        )
        self.assertEqual(response.status_code, 403)

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_index_instance_document_node_view_with_full_access(self):
        self.grant_access(
            obj=self._test_index_template,
            permission=permission_index_instance_view
        )
        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._clear_events()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self._test_index_instance_node
        )
        self.assertContains(
            response=response, text=TEST_INDEX_TEMPLATE_LABEL,
            status_code=200
        )
        self.assertContains(
            response=response, text=self._test_document.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_index_instance_trashed_document_node_view_with_full_access(self):
        self.grant_access(
            obj=self._test_index_template,
            permission=permission_index_instance_view
        )
        self.grant_access(
            obj=self._test_document, permission=permission_document_view
        )

        self._test_document.delete()

        self._clear_events()

        response = self._request_test_index_instance_node_view(
            index_instance_node=self._test_index_instance_node
        )

        self.assertContains(
            response=response, text=TEST_INDEX_TEMPLATE_LABEL,
            status_code=200
        )
        self.assertNotContains(
            response=response, text=self._test_document.label,
            status_code=200
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



import logging

from rest_framework import status

from mayan.apps.converter.api_view_mixins import APIImageViewMixin
from mayan.apps.rest_api import generics

from ..classes import DocumentVersionModification
from ..permissions import (
    permission_document_version_create, permission_document_version_delete,
    permission_document_version_edit, permission_document_version_view
)
from ..serializers.document_version_serializers import (
    DocumentVersionModificationSerializer,
    DocumentVersionModificationExecuteSerializer, DocumentVersionSerializer,
    DocumentVersionPageSerializer
)

from .api_view_mixins import (
    ParentObjectDocumentAPIViewMixin, ParentObjectDocumentVersionAPIViewMixin
)

logger = logging.getLogger(name=__name__)


class APIDocumentVersionDetailView(
    ParentObjectDocumentAPIViewMixin, generics.RetrieveUpdateDestroyAPIView
):
    """
    delete: Delete the selected document version.
    get: Returns the selected document version details.
    patch: Edit the properties of the selected document version.
    put: Edit the properties of the selected document version.
    """
    lookup_url_kwarg = 'document_version_id'
    mayan_object_permissions = {
        'DELETE': (permission_document_version_delete,),
        'GET': (permission_document_version_view,),
        'PATCH': (permission_document_version_edit,),
        'PUT': (permission_document_version_edit,)
    }
    serializer_class = DocumentVersionSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }

    def get_source_queryset(self):
        return self.get_document().versions.all()


class APIDocumentVersionListView(
    ParentObjectDocumentAPIViewMixin, generics.ListCreateAPIView
):
    """
    get: Return a list of the selected document's versions.
    post: Create a new document version.
    """
    ordering_fields = ('active', 'comment', 'id')
    serializer_class = DocumentVersionSerializer

    def get_instance_extra_data(self):
        # This method is only called during POST, therefore filter only by
        # edit permission.
        return {
            '_event_actor': self.request.user,
            'document': self.get_document(
                permission=permission_document_version_create
            )
        }

    def get_source_queryset(self):
        # This method is only called during GET, therefore filter only by
        # the view permission.
        return self.get_document(
            permission=permission_document_version_view
        ).versions.all()


class APIDocumentVersionModificationView(
    ParentObjectDocumentAPIViewMixin, generics.ObjectActionAPIView
):
    """
    post: Execute a modification backend on the selected document version.
    """
    action_response_status = status.HTTP_202_ACCEPTED
    lookup_url_kwarg = 'document_version_id'
    mayan_object_permissions = {
        'POST': (permission_document_version_edit,),
    }
    serializer_class = DocumentVersionModificationExecuteSerializer

    def get_source_queryset(self):
        return self.get_document().versions.all()

    def object_action(self, obj, request, serializer):
        DocumentVersionModification.get(
            name=serializer.validated_data['backend_id']
        ).execute(
            document_version=obj, user=request.user
        )


class APIDocumentVersionModificationBackendListView(generics.ListAPIView):
    """
    get: Returns a list of the available document version modification backends.
    """
    serializer_class = DocumentVersionModificationSerializer

    def get_serializer_context(self):
        return {
            'format': self.format_kwarg,
            'request': self.request,
            'view': self
        }

    def get_source_queryset(self):
        return DocumentVersionModification.get_all()


class APIDocumentVersionPageDetailView(
    ParentObjectDocumentVersionAPIViewMixin,
    generics.RetrieveUpdateDestroyAPIView
):
    """
    delete: Delete the selected document version page.
    get: Returns the selected document version page details.
    patch: Edit the properties of the selected document version page.
    put: Edit the properties of the selected document version page.
    """
    lookup_url_kwarg = 'document_version_page_id'
    mayan_object_permissions = {
        'DELETE': (permission_document_version_edit,),
        'GET': (permission_document_version_view,),
        'PATCH': (permission_document_version_edit,),
        'PUT': (permission_document_version_edit,)
    }
    serializer_class = DocumentVersionPageSerializer

    def get_instance_extra_data(self):
        return {
            '_event_actor': self.request.user
        }

    def get_source_queryset(self):
        return self.get_document_version().pages.all()


class APIDocumentVersionPageImageView(
    APIImageViewMixin, ParentObjectDocumentVersionAPIViewMixin,
    generics.RetrieveAPIView
):
    """
    get: Returns an image representation of the selected document version page.
    """
    lookup_url_kwarg = 'document_version_page_id'
    mayan_object_permissions = {
        'GET': (permission_document_version_view,)
    }

    def get_source_queryset(self):
        return self.get_document_version().pages.all()


class APIDocumentVersionPageListView(
    ParentObjectDocumentVersionAPIViewMixin, generics.ListCreateAPIView
):
    """
    get: Returns an list of the pages for the selected document version.
    post: Create a new document version page.
    """
    lookup_url_kwarg = 'document_version_page_id'
    serializer_class = DocumentVersionPageSerializer

    def get_instance_extra_data(self):
        # This method is only called during POST, therefore filter only by
        # edit permission.
        return {
            '_event_actor': self.request.user,
            'document_version': self.get_document_version(
                permission=permission_document_version_edit
            )
        }

    def get_source_queryset(self):
        # This method is only called during GET, therefore filter only by
        # the view permission.
        return self.get_document_version(
            permission=permission_document_version_view
        ).pages.all()



from django.utils.module_loading import import_string
from django.utils.translation import ugettext_lazy as _

from . import serializers


class DynamicSerializerField(serializers.ReadOnlyField):
    serializers = {}

    @classmethod
    def add_serializer(cls, klass, serializer_class):
        if isinstance(klass, str):
            klass = import_string(dotted_path=klass)

        if isinstance(serializer_class, str):
            serializer_class = import_string(dotted_path=serializer_class)

        cls.serializers[klass] = serializer_class

    def to_representation(self, value):
        for klass, serializer_class in self.serializers.items():
            if isinstance(value, klass):
                return serializer_class(
                    context={
                        'format': self.context['format'],
                        'request': self.context['request']
                    }
                ).to_representation(instance=value)

        return _('Unable to find serializer class for: %s') % value



from django.db import models
from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.validators import YAMLValidator
from mayan.apps.databases.model_mixins import ExtraDataModelMixin
from mayan.apps.events.decorators import method_event
from mayan.apps.events.event_managers import (
    EventManagerMethodAfter, EventManagerSave
)
from mayan.apps.events.models import StoredEventType

from ..events import event_workflow_template_edited
from ..literals import FIELD_TYPE_CHOICES, WIDGET_CLASS_CHOICES

from .workflow_models import Workflow
from .workflow_state_models import WorkflowState
from .workflow_transition_model_mixins import (
    WorkflowTransitionBusinessLogicMixin,
    WorkflowTransitionFieldBusinessLogicMixin,
    WorkflowTransitionTriggerEventBusinessLogicMixin
)

__all__ = (
    'WorkflowTransition', 'WorkflowTransitionField',
    'WorkflowTransitionTriggerEvent'
)


class WorkflowTransition(
    ExtraDataModelMixin, WorkflowTransitionBusinessLogicMixin, models.Model
):
    workflow = models.ForeignKey(
        on_delete=models.CASCADE, related_name='transitions', to=Workflow,
        verbose_name=_('Workflow')
    )
    label = models.CharField(
        help_text=_('A short text to describe the transition.'),
        max_length=255, verbose_name=_('Label')
    )
    origin_state = models.ForeignKey(
        on_delete=models.CASCADE, related_name='origin_transitions',
        to=WorkflowState, verbose_name=_('Origin state')
    )
    destination_state = models.ForeignKey(
        on_delete=models.CASCADE, related_name='destination_transitions',
        to=WorkflowState, verbose_name=_('Destination state')
    )
    condition = models.TextField(
        blank=True, help_text=_(
            'The condition that will determine if this transition '
            'is enabled or not. The condition is evaluated against the '
            'workflow instance. Conditions that do not return any value, '
            'that return the Python logical None, or an empty string (\'\') '
            'are considered to be logical false, any other value is '
            'considered to be the logical true.'
        ), verbose_name=_('Condition')
    )

    class Meta:
        ordering = ('label',)
        unique_together = (
            'workflow', 'label', 'origin_state', 'destination_state'
        )
        verbose_name = _('Workflow transition')
        verbose_name_plural = _('Workflow transitions')

    def __str__(self):
        return self.label

    @method_event(
        action_object='self',
        event_manager_class=EventManagerMethodAfter,
        event=event_workflow_template_edited,
        target='workflow'
    )
    def delete(self, *args, **kwargs):
        return super().delete(*args, **kwargs)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'self',
            'event': event_workflow_template_edited,
            'target': 'workflow'
        },
        edited={
            'action_object': 'self',
            'event': event_workflow_template_edited,
            'target': 'workflow'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class WorkflowTransitionField(
    ExtraDataModelMixin, WorkflowTransitionFieldBusinessLogicMixin,
    models.Model
):
    transition = models.ForeignKey(
        on_delete=models.CASCADE, related_name='fields',
        to=WorkflowTransition, verbose_name=_('Transition')
    )
    field_type = models.PositiveIntegerField(
        choices=FIELD_TYPE_CHOICES, verbose_name=_('Type')
    )
    name = models.CharField(
        help_text=_(
            'The name that will be used to identify this field in other parts '
            'of the workflow system.'
        ), max_length=128, verbose_name=_('Internal name')
    )
    label = models.CharField(
        help_text=_(
            'The field name that will be shown on the user interface.'
        ), max_length=128, verbose_name=_('Label'))
    help_text = models.TextField(
        blank=True, help_text=_(
            'An optional message that will help users better understand the '
            'purpose of the field and data to provide.'
        ), verbose_name=_('Help text')
    )
    required = models.BooleanField(
        default=False, help_text=_(
            'Whether this fields needs to be filled out or not to proceed.'
        ), verbose_name=_('Required')
    )
    widget = models.PositiveIntegerField(
        blank=True, choices=WIDGET_CLASS_CHOICES, help_text=_(
            'An optional class to change the default presentation of the field.'
        ), null=True, verbose_name=_('Widget class')
    )
    widget_kwargs = models.TextField(
        blank=True, help_text=_(
            'A group of keyword arguments to customize the widget. '
            'Use YAML format.'
        ), validators=[YAMLValidator()],
        verbose_name=_('Widget keyword arguments')
    )

    class Meta:
        unique_together = ('transition', 'name')
        verbose_name = _('Workflow transition field')
        verbose_name_plural = _('Workflow transition fields')

    def __str__(self):
        return self.label

    @method_event(
        action_object='self',
        event_manager_class=EventManagerMethodAfter,
        event=event_workflow_template_edited,
        target='transition.workflow'
    )
    def delete(self, *args, **kwargs):
        return super().delete(*args, **kwargs)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'self',
            'event': event_workflow_template_edited,
            'target': 'transition.workflow'
        },
        edited={
            'action_object': 'self',
            'event': event_workflow_template_edited,
            'target': 'transition.workflow'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)


class WorkflowTransitionTriggerEvent(
    ExtraDataModelMixin, WorkflowTransitionTriggerEventBusinessLogicMixin,
    models.Model
):
    transition = models.ForeignKey(
        on_delete=models.CASCADE, related_name='trigger_events',
        to=WorkflowTransition, verbose_name=_('Transition')
    )
    event_type = models.ForeignKey(
        on_delete=models.CASCADE, to=StoredEventType,
        verbose_name=_('Event type')
    )

    class Meta:
        ordering = ('event_type__name',)
        unique_together = ('transition', 'event_type')
        verbose_name = _('Workflow transition trigger event')
        verbose_name_plural = _('Workflow transitions trigger events')

    def __str__(self):
        return str(self.transition)

    @method_event(
        action_object='self',
        event_manager_class=EventManagerMethodAfter,
        event=event_workflow_template_edited,
        target='transition.workflow'
    )
    def delete(self, *args, **kwargs):
        return super().delete(*args, **kwargs)

    @method_event(
        event_manager_class=EventManagerSave,
        created={
            'action_object': 'self',
            'event': event_workflow_template_edited,
            'target': 'transition.workflow'
        },
        edited={
            'action_object': 'self',
            'event': event_workflow_template_edited,
            'target': 'transition.workflow'
        }
    )
    def save(self, *args, **kwargs):
        return super().save(*args, **kwargs)



from mayan.apps.appearance.classes import Icon

# Clear

icon_event_list_clear = Icon(
    driver_name='fontawesome', symbol='broom'
)
icon_object_event_list_clear = Icon(
    driver_name='fontawesome', symbol='broom'
)
icon_verb_event_list_clear = Icon(
    driver_name='fontawesome', symbol='broom'
)

# Export

icon_event_list_export = Icon(
    driver_name='fontawesome', symbol='file-export'
)
icon_object_event_list_export = Icon(
    driver_name='fontawesome', symbol='file-export'
)
icon_verb_event_list_export = Icon(
    driver_name='fontawesome', symbol='file-export'
)

# List

icon_event_list = Icon(driver_name='fontawesome', symbol='list-ol')
icon_object_event_list = Icon(driver_name='fontawesome', symbol='list-ol')
icon_verb_event_list = Icon(driver_name='fontawesome', symbol='list-ol')

# Notifications

icon_notification_list = Icon(
    driver_name='fontawesome', symbol='bell'
)
icon_notification_mark_read = Icon(
    driver_name='fontawesome', symbol='eye'
)
icon_notification_mark_read_all = Icon(
    driver_name='fontawesome', symbol='eye'
)

# Subscriptions

icon_event_types_subscriptions_list = Icon(
    driver_name='fontawesome-dual', primary_symbol='list-ol',
    secondary_symbol='rss'
)
icon_object_event_type_user_subscription_list = Icon(
    driver_name='fontawesome', symbol='rss'
)
icon_user_object_subscriptions_list = Icon(
    driver_name='fontawesome', symbol='rss'
)



from ...settings import setting_results_limit


class BackendSearchTestMixin:
    _test_search_model = None

    def _do_backend_search(
        self, field_name, query_type, value, limit=None,
        is_quoted_value=False, is_raw_value=False, _skip_refresh=None
    ):
        limit = limit or setting_results_limit.value
        search_field = self._test_search_model.get_search_field(
            field_name=field_name
        )

        return self._test_search_backend._search(
            is_quoted_value=is_quoted_value, is_raw_value=is_raw_value,
            limit=limit, query_type=query_type, search_field=search_field,
            value=value, _skip_refresh=_skip_refresh
        )

    def _do_search(self, query):
        self._test_search_backend.refresh()

        return self._test_search_backend.search(
            search_model=self._test_search_model, query=query,
            user=self._test_case_user
        )



from mozilla_django_oidc.auth import OIDCAuthenticationBackend


class DjangoAuthenticationBackendOIDC(OIDCAuthenticationBackend):
    def create_user(self, claims):
        user = super().create_user(claims=claims)

        user._event_ignore = True

        user.first_name = claims.get('given_name', '')
        user.last_name = claims.get('family_name', '')
        user.save(update_fields=('first_name', 'last_name'))

        return user

    def update_user(self, user, claims):
        user = super().update_user(user=user, claims=claims)

        user._event_ignore = True

        user.first_name = claims.get('given_name', '')
        user.last_name = claims.get('family_name', '')
        user.save(update_fields=('first_name', 'last_name'))

        return user



import logging

from mayan.apps.testing.tests.base import BaseTestCase
from mayan.apps.testing.tests.utils import mute_stdout

from ..models import AutoAdminSingleton
from ..settings import setting_username

from .literals import TEST_ADMIN_USER_PASSWORD


class AutoAdminHandlerTestCase(BaseTestCase):
    def test_post_admin_creation(self):
        logging.disable(logging.INFO)

        with mute_stdout():
            AutoAdminSingleton.objects.create_autoadmin()

        self.assertEqual(
            AutoAdminSingleton.objects.get().account.username,
            setting_username.value
        )

        user = AutoAdminSingleton.objects.get().account

        user.set_password(TEST_ADMIN_USER_PASSWORD)
        user.save(update_fields=('password',))

        self.assertEqual(AutoAdminSingleton.objects.get().account, None)

    def test_double_creation(self):
        with mute_stdout():
            AutoAdminSingleton.objects.create_autoadmin()

        self.assertEqual(AutoAdminSingleton.objects.count(), 1)

        logging.disable(logging.ERROR)

        AutoAdminSingleton.objects.create_autoadmin()
        self.assertEqual(AutoAdminSingleton.objects.count(), 1)



from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('mailer', '0003_auto_20170703_1535')
    ]

    operations = [
        migrations.AlterField(
            model_name='usermailer',
            name='default',
            field=models.BooleanField(
                default=True, help_text='If default, this mailing profile '
                'will be pre-selected on the document mailing form.',
                verbose_name='Default'
            )
        ),
    ]



from django.shortcuts import get_object_or_404
from django.template import RequestContext
from django.utils.html import mark_safe
from django.utils.translation import ugettext_lazy as _

from mayan.apps.acls.models import AccessControlList
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.views.document_views import DocumentListView
from mayan.apps.views.generics import SingleObjectListView
from mayan.apps.views.view_mixins import ExternalObjectViewMixin

from ..html_widgets import node_tree
from ..icons import (
    icon_index, icon_document_index_instance_list, icon_index_instance_list,
    icon_index_instance_node_with_documents
)
from ..links import link_index_template_create
from ..models import (
    DocumentIndexInstanceNode, IndexInstance, IndexInstanceNode
)
from ..permissions import permission_index_instance_view


class IndexInstanceListView(SingleObjectListView):
    object_permission = permission_index_instance_view
    view_icon = icon_index_instance_list

    def get_extra_context(self):
        return {
            'hide_links': True,
            'hide_object': True,
            'no_results_icon': icon_index,
            'no_results_main_link': link_index_template_create.resolve(
                context=RequestContext(request=self.request)
            ),
            'no_results_text': _(
                'This could mean that no index templates have been '
                'created or that there are index templates '
                'but they are no properly defined.'
            ),
            'no_results_title': _('There are no index instances available.'),
            'title': _('Indexes')
        }

    def get_source_queryset(self):
        queryset = IndexInstance.objects.filter(enabled=True)
        return queryset.filter(
            index_template_nodes__index_instance_nodes__isnull=False
        ).distinct()


class IndexInstanceNodeView(DocumentListView):
    template_name = 'document_indexing/node_details.html'
    view_icon = icon_index_instance_node_with_documents

    def dispatch(self, request, *args, **kwargs):
        self.index_instance_node = self.get_index_instance_node()

        if self.index_instance_node:
            if self.index_instance_node.index_template_node.link_documents:
                return super().dispatch(request=request, *args, **kwargs)

        return SingleObjectListView.dispatch(
            self, request=request, *args, **kwargs
        )

    def get_document_queryset(self):
        if self.index_instance_node:
            if self.index_instance_node.index_template_node.link_documents:
                return self.index_instance_node._get_documents()

        return Document.valid.none()

    def get_extra_context(self):
        context = super().get_extra_context()
        context.update(
            {
                'column_class': 'col-xs-12 col-sm-6 col-md-4 col-lg-3',
                'navigation': mark_safe(
                    s=_('Navigation: %s') % node_tree(
                        node=self.index_instance_node, user=self.request.user
                    )
                ),
                'object': self.index_instance_node,
                'title': _(
                    'Contents for index: %s'
                ) % self.index_instance_node.get_full_path()
            }
        )

        if self.index_instance_node and not self.index_instance_node.index_template_node.link_documents:
            context.update(
                {
                    'hide_object': True,
                    'list_as_items': False
                }
            )

        return context

    def get_index_instance_node(self):
        instance = get_object_or_404(
            klass=IndexInstanceNode, pk=self.kwargs['index_instance_node_id']
        )
        AccessControlList.objects.check_access(
            obj=instance, permissions=(
                permission_index_instance_view,
            ), user=self.request.user
        )
        return instance

    def get_source_queryset(self):
        if self.index_instance_node:
            if self.index_instance_node.index_template_node.link_documents:
                return super().get_source_queryset()
            else:
                self.object_permission = None
                return self.index_instance_node.get_children().order_by(
                    'value'
                )
        else:
            self.object_permission = None
            return IndexInstanceNode.objects.none()


class DocumentIndexInstanceNodeListView(
    ExternalObjectViewMixin, SingleObjectListView
):
    """
    Show a list of indexes where the current document can be found.
    """
    external_object_permission = permission_index_instance_view
    external_object_pk_url_kwarg = 'document_id'
    external_object_queryset = Document.valid.all()
    object_permission = permission_index_instance_view
    view_icon = icon_document_index_instance_list

    def get_extra_context(self):
        return {
            'hide_object': True,
            'no_results_icon': icon_index,
            'no_results_text': _(
                'Assign the document type of this document '
                'to an index to have it appear in instances of '
                'those indexes organization units. '
            ),
            'no_results_title': _(
                'This document is not in any index'
            ),
            'object': self.external_object,
            'title': _(
                'Indexes nodes containing document: %s'
            ) % self.external_object
        }

    def get_source_queryset(self):
        return DocumentIndexInstanceNode.objects.get_for(
            document=self.external_object
        )



from django.db import migrations, models
import django.db.models.deletion

import mptt.fields


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('documents', '0034_auto_20160509_2321'),
    ]

    operations = [
        migrations.CreateModel(
            name='Cabinet',
            fields=[
                (
                    'id', models.AutoField(
                        auto_created=True, primary_key=True, serialize=False,
                        verbose_name='ID'
                    )
                ),
                (
                    'label', models.CharField(
                        max_length=128, verbose_name='Label'
                    )
                ),
                (
                    'lft', models.PositiveIntegerField(
                        db_index=True, editable=False
                    )
                ),
                (
                    'rght', models.PositiveIntegerField(
                        db_index=True, editable=False
                    )
                ),
                (
                    'tree_id', models.PositiveIntegerField(
                        db_index=True, editable=False
                    )
                ),
                (
                    'level', models.PositiveIntegerField(
                        db_index=True, editable=False
                    )
                ),
                (
                    'documents', models.ManyToManyField(
                        blank=True, related_name='cabinets',
                        to='documents.Document', verbose_name='Documents'
                    )
                ),
                (
                    'parent', mptt.fields.TreeForeignKey(
                        blank=True, null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name='children', to='cabinets.Cabinet'
                    )
                ),
            ],
            options={
                'ordering': ('parent__label', 'label'),
                'verbose_name': 'Cabinet',
                'verbose_name_plural': 'Cabinets',
            },
        ),
        migrations.CreateModel(
            name='DocumentCabinet',
            fields=[
            ],
            options={
                'verbose_name': 'Document cabinet',
                'proxy': True,
                'verbose_name_plural': 'Document cabinets',
            },
            bases=('cabinets.cabinet',),
        ),
        migrations.AlterUniqueTogether(
            name='cabinet',
            unique_together={('parent', 'label')},
        ),
    ]



import logging

from django.utils.translation import ugettext_lazy as _

from mayan.apps.common.apps import MayanAppConfig

from .patches import patch_HttpRequest

logger = logging.getLogger(name=__name__)


class OrganizationsApp(MayanAppConfig):
    app_namespace = 'organizations'
    app_url = 'organizations'
    has_tests = True
    name = 'mayan.apps.organizations'
    verbose_name = _('Organizations')

    def ready(self):
        super().ready()

        patch_HttpRequest()



from django.utils.translation import ugettext_lazy as _

from mayan.apps.permissions import PermissionNamespace

namespace = PermissionNamespace(label=_('Credentials'), name='credentials')

permission_credential_create = namespace.add_permission(
    label=_('Create credentials'), name='credential_create'
)
permission_credential_delete = namespace.add_permission(
    label=_('Delete credentials'), name='credential_delete'
)
permission_credential_edit = namespace.add_permission(
    label=_('Edit credentials'), name='credential_edit'
)
permission_credential_use = namespace.add_permission(
    label=_('Use credential'), name='credential_use'
)
permission_credential_view = namespace.add_permission(
    label=_('View credentials'), name='credential_view'
)



from datetime import timedelta

from django.utils import timezone

from mayan.apps.testing.tests.base import BaseTestCase

from ..models import Announcement

from .mixins import AnnouncementTestMixin


class AnnouncementModelTestCase(AnnouncementTestMixin, BaseTestCase):
    def setUp(self):
        super().setUp()
        self._create_test_announcement()

    def test_basic(self):
        queryset = Announcement.objects.get_for_now()

        self.assertEqual(queryset.exists(), True)

    def test_end_datetime(self):
        self._test_announcement.start_datetime = timezone.now() - timedelta(days=2)
        self._test_announcement.end_datetime = timezone.now() - timedelta(days=1)
        self._test_announcement.save()

        queryset = Announcement.objects.get_for_now()

        self.assertEqual(queryset.exists(), False)

    def test_enable(self):
        self._test_announcement.enabled = False
        self._test_announcement.save()

        queryset = Announcement.objects.get_for_now()

        self.assertEqual(queryset.exists(), False)

    def test_start_datetime(self):
        self._test_announcement.start_datetime = timezone.now() - timedelta(days=1)
        self._test_announcement.save()

        queryset = Announcement.objects.get_for_now()

        self.assertEqual(queryset.first(), self._test_announcement)

    def test_method_get_absolute_url(self):
        self.assertTrue(self._test_announcement.get_absolute_url())



from pathlib import Path

from mayan.apps.documents.events import (
    event_document_created, event_document_file_created,
    event_document_file_edited, event_document_version_created,
    event_document_version_page_created
)
from mayan.apps.documents.models.document_models import Document
from mayan.apps.documents.tests.base import GenericDocumentTestCase
from mayan.apps.documents.tests.literals import (
    TEST_DOCUMENT_SMALL_CHECKSUM, TEST_FILE_COMPRESSED_PATH,
    TEST_FILE_SMALL_PATH
)
from mayan.apps.source_compressed.source_backends.literals import (
    SOURCE_UNCOMPRESS_CHOICE_ALWAYS, SOURCE_UNCOMPRESS_CHOICE_NEVER
)
from mayan.apps.sources.exceptions import SourceActionException

from .mixins import WatchStorageSourceTestMixin


class WatchStorageSourceBackendActionDocumentUploadTestCase(
    WatchStorageSourceTestMixin, GenericDocumentTestCase
):
    _test_source_create_auto = False
    auto_upload_test_document = False

    def test_basic(self):
        self._test_source_create()

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_compressed_always(self):
        self._test_source_create(
            extra_data={'uncompress': SOURCE_UNCOMPRESS_CHOICE_ALWAYS}
        )

        self.copy_test_source_file(
            source_path=TEST_FILE_COMPRESSED_PATH
        )

        test_document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        self.assertEqual(Document.objects.count(), test_document_count + 2)

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 11)

        test_documents = Document.objects.all()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_documents[0])
        self.assertEqual(events[0].target, test_documents[0])
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_documents[0])
        self.assertEqual(events[1].actor, test_documents[0].file_latest)
        self.assertEqual(events[1].target, test_documents[0].file_latest)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_documents[0])
        self.assertEqual(events[2].actor, test_documents[0].file_latest)
        self.assertEqual(events[2].target, test_documents[0].file_latest)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_documents[0])
        self.assertEqual(events[3].actor, test_documents[0].version_active)
        self.assertEqual(events[3].target, test_documents[0].version_active)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(
            events[4].action_object, test_documents[0].version_active
        )
        self.assertEqual(
            events[4].actor, test_documents[0].version_active.pages.first()
        )
        self.assertEqual(
            events[4].target, test_documents[0].version_active.pages.first()
        )
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

        self.assertEqual(events[5].action_object, self._test_document_type)
        self.assertEqual(events[5].actor, test_documents[1])
        self.assertEqual(events[5].target, test_documents[1])
        self.assertEqual(events[5].verb, event_document_created.id)

        self.assertEqual(events[6].action_object, test_documents[1])
        self.assertEqual(events[6].actor, test_documents[1].file_latest)
        self.assertEqual(events[6].target, test_documents[1].file_latest)
        self.assertEqual(events[6].verb, event_document_file_created.id)

        self.assertEqual(events[7].action_object, test_documents[1])
        self.assertEqual(events[7].actor, test_documents[1].file_latest)
        self.assertEqual(events[7].target, test_documents[1].file_latest)
        self.assertEqual(events[7].verb, event_document_file_edited.id)

        self.assertEqual(events[8].action_object, test_documents[1])
        self.assertEqual(events[8].actor, test_documents[1].version_active)
        self.assertEqual(events[8].target, test_documents[1].version_active)
        self.assertEqual(events[8].verb, event_document_version_created.id)

        self.assertEqual(
            events[9].action_object, test_documents[1].version_active
        )
        self.assertEqual(
            events[9].actor, test_documents[1].version_active.pages.first()
        )
        self.assertEqual(
            events[9].target, test_documents[1].version_active.pages.first()
        )
        self.assertEqual(
            events[9].verb, event_document_version_page_created.id
        )

        self.assertEqual(
            events[10].action_object, test_documents[1].version_active
        )
        self.assertEqual(
            events[10].actor, test_documents[1].version_active.pages.last()
        )
        self.assertEqual(
            events[10].target, test_documents[1].version_active.pages.last()
        )
        self.assertEqual(
            events[10].verb, event_document_version_page_created.id
        )

    def test_compressed_never(self):
        self._silence_logger(name='mayan.apps.converter.backends')

        self._test_source_create(
            extra_data={'uncompress': SOURCE_UNCOMPRESS_CHOICE_NEVER}
        )

        self.copy_test_source_file(
            source_path=TEST_FILE_COMPRESSED_PATH
        )

        test_document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        self.assertEqual(Document.objects.count(), test_document_count + 1)

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 3)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_version)
        self.assertEqual(events[2].target, test_document_version)
        self.assertEqual(events[2].verb, event_document_version_created.id)

    def test_dry_run_false(self):
        self._test_source_create()

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(
            action_name='document_upload', extra_data={'dry_run': False}
        )

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_dry_run_none(self):
        self._test_source_create()

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(
            action_name='document_upload', extra_data={'dry_run': None}
        )

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_dry_run_true(self):
        self._test_source_create()

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(
            action_name='document_upload', extra_data={'dry_run': True}
        )

        self.assertEqual(Document.objects.count(), document_count + 1)
        self.assertEqual(
            Document.objects.first().file_latest.checksum,
            TEST_DOCUMENT_SMALL_CHECKSUM
        )

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_regular_expression_exclude_false(self):
        path = Path(TEST_FILE_SMALL_PATH)

        self._test_source_create(
            extra_data={'exclude_regex': path.name}
        )

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        self.assertEqual(Document.objects.count(), document_count)

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_regular_expression_exclude_true(self):
        self._test_source_create(
            extra_data={'exclude_regex': ''}
        )

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        self.assertEqual(Document.objects.count(), document_count + 1)

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )

    def test_regular_expression_include_false(self):
        self._test_source_create(
            extra_data={'include_regex': '_____.*'}
        )

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        self.assertEqual(Document.objects.count(), document_count)

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_regular_expression_include_true(self):
        path = Path(TEST_FILE_SMALL_PATH)

        self._test_source_create(
            extra_data={'include_regex': path.name}
        )

        self.copy_test_source_file()

        document_count = Document.objects.count()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='document_upload')

        self.assertEqual(Document.objects.count(), document_count + 1)

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 5)

        test_document = Document.objects.first()
        test_document_file = test_document.file_latest
        test_document_version = test_document.version_active
        test_document_version_page = test_document_version.pages.first()

        self.assertEqual(events[0].action_object, self._test_document_type)
        self.assertEqual(events[0].actor, test_document)
        self.assertEqual(events[0].target, test_document)
        self.assertEqual(events[0].verb, event_document_created.id)

        self.assertEqual(events[1].action_object, test_document)
        self.assertEqual(events[1].actor, test_document_file)
        self.assertEqual(events[1].target, test_document_file)
        self.assertEqual(events[1].verb, event_document_file_created.id)

        self.assertEqual(events[2].action_object, test_document)
        self.assertEqual(events[2].actor, test_document_file)
        self.assertEqual(events[2].target, test_document_file)
        self.assertEqual(events[2].verb, event_document_file_edited.id)

        self.assertEqual(events[3].action_object, test_document)
        self.assertEqual(events[3].actor, test_document_version)
        self.assertEqual(events[3].target, test_document_version)
        self.assertEqual(events[3].verb, event_document_version_created.id)

        self.assertEqual(events[4].action_object, test_document_version)
        self.assertEqual(events[4].actor, test_document_version_page)
        self.assertEqual(events[4].target, test_document_version_page)
        self.assertEqual(
            events[4].verb, event_document_version_page_created.id
        )


class WatchStorageSourceBackendActionFileDeleteTestCase(
    WatchStorageSourceTestMixin, GenericDocumentTestCase
):
    _test_source_create_auto = False
    auto_upload_test_document = False

    def test_basic(self):
        self._test_source_create()

        self.copy_test_source_file()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        self._execute_test_source_action(action_name='file_delete')

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count - 1
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)

    def test_nonexistent_file(self):
        self._test_source_create()

        self.copy_test_source_file()

        self._test_source_stored_test_file.unlink()

        test_source_stored_file_count = len(
            self.get_test_source_stored_file_list()
        )

        self._clear_events()

        with self.assertRaises(expected_exception=SourceActionException):
            self._execute_test_source_action(action_name='file_delete')

        self.assertEqual(
            len(
                self.get_test_source_stored_file_list()
            ), test_source_stored_file_count
        )

        events = self._get_test_events()
        self.assertEqual(events.count(), 0)



from django import forms
from django.forms.formsets import formset_factory
from django.utils.translation import ugettext_lazy as _

from mayan.apps.backends.forms import FormDynamicModelBackend
from mayan.apps.converter.fields import ImageField
from mayan.apps.templating.fields import ModelTemplateField
from mayan.apps.views.forms import FilteredSelectionForm

from .classes import WorkflowAction
from .models import (
    Workflow, WorkflowStateEscalation, WorkflowInstance, WorkflowState,
    WorkflowStateAction, WorkflowTransition
)


class WorkflowActionSelectionForm(forms.Form):
    klass = forms.ChoiceField(
        choices=(), help_text=_('The action type for this action entry.'),
        label=_('Action'), widget=forms.Select(
            attrs={'class': 'select2'}
        )
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['klass'].choices = WorkflowAction.get_choices()


class WorkflowForm(forms.ModelForm):
    class Meta:
        fields = ('label', 'internal_name', 'auto_launch')
        model = Workflow


class WorkflowMultipleSelectionForm(FilteredSelectionForm):
    class Meta:
        allow_multiple = True
        field_name = 'workflows'
        label = _('Workflows')
        required = False
        widget_attributes = {'class': 'select2'}


class WorkflowStateActionDynamicForm(FormDynamicModelBackend):
    class Meta:
        fields = ('label', 'when', 'enabled', 'condition', 'backend_data')
        model = WorkflowStateAction
        widgets = {'backend_data': forms.widgets.HiddenInput}

    def __init__(self, request, user=None, *args, **kwargs):
        self.request = request
        self.user = user
        result = super().__init__(*args, **kwargs)

        self.fields['condition'] = ModelTemplateField(
            initial_help_text=self.fields['condition'].help_text,
            label=self.fields['condition'].label, model=WorkflowInstance,
            model_variable='workflow_instance', required=False
        )

        return result


class WorkflowStateEscalationForm(forms.ModelForm):
    def __init__(self, workflow_template_state, *args, **kwargs):
        self.workflow_template_state = workflow_template_state
        super().__init__(*args, **kwargs)

        self.fields[
            'transition'
        ].queryset = self.workflow_template_state.workflow.transitions.filter(
            origin_state=self.workflow_template_state
        )

        self.fields['condition'] = ModelTemplateField(
            initial_help_text=self.fields['condition'].help_text,
            label=self.fields['condition'].label, model=WorkflowInstance,
            model_variable='workflow_instance', required=False
        )

    class Meta:
        fields = (
            'enabled', 'transition', 'priority', 'unit', 'amount',
            'condition'
        )
        model = WorkflowStateEscalation


class WorkflowStateForm(forms.ModelForm):
    class Meta:
        fields = ('initial', 'label', 'completion')
        model = WorkflowState


class WorkflowTransitionForm(forms.ModelForm):
    def __init__(self, workflow, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields[
            'origin_state'
        ].queryset = self.fields[
            'origin_state'
        ].queryset.filter(workflow=workflow)

        self.fields[
            'destination_state'
        ].queryset = self.fields[
            'destination_state'
        ].queryset.filter(workflow=workflow)

        self.fields['condition'] = ModelTemplateField(
            initial_help_text=self.fields['condition'].help_text,
            label=self.fields['condition'].label, model=WorkflowInstance,
            model_variable='workflow_instance', required=False
        )

    class Meta:
        fields = ('label', 'origin_state', 'destination_state', 'condition')
        model = WorkflowTransition


class WorkflowTransitionTriggerEventRelationshipForm(forms.Form):
    namespace = forms.CharField(
        label=_('Namespace'), required=False,
        widget=forms.TextInput(
            attrs={'readonly': 'readonly'}
        )
    )
    label = forms.CharField(
        label=_('Label'), required=False,
        widget=forms.TextInput(
            attrs={'readonly': 'readonly'}
        )
    )
    relationship = forms.ChoiceField(
        choices=(
            ('no', _('No')),
            ('yes', _('Yes')),
        ), label=_('Enabled'), widget=forms.RadioSelect()
    )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['namespace'].initial = self.initial['event_type'].namespace
        self.fields['label'].initial = self.initial['event_type'].label

        relationship = self.initial['transition'].trigger_events.filter(
            event_type=self.initial['event_type']
        )

        if relationship.exists():
            self.fields['relationship'].initial = 'yes'
        else:
            self.fields['relationship'].initial = 'no'

    def save(self):
        relationship = self.initial['transition'].trigger_events.filter(
            event_type=self.initial['event_type']
        )

        if self.cleaned_data['relationship'] == 'no':
            relationship.delete()
        elif self.cleaned_data['relationship'] == 'yes':
            if not relationship.exists():
                self.initial['transition'].trigger_events.create(
                    event_type=self.initial['event_type']
                )


WorkflowTransitionTriggerEventRelationshipFormSet = formset_factory(
    form=WorkflowTransitionTriggerEventRelationshipForm, extra=0
)


class WorkflowInstanceTransitionSelectForm(forms.Form):
    def __init__(self, user, workflow_instance, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields[
            'transition'
        ].queryset = workflow_instance.get_transition_choices(user=user)

    transition = forms.ModelChoiceField(
        help_text=_('Select a transition to execute in the next step.'),
        label=_('Transition'), queryset=WorkflowTransition.objects.none()
    )


class WorkflowPreviewForm(forms.Form):
    workflow = ImageField(
        image_alt_text=_('Workflow template preview image')
    )

    def __init__(self, instance=None, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['workflow'].initial = instance



from django import forms
from django.utils.translation import ugettext_lazy as _

from mayan.apps.views.forms import Form

from .literals import MATCH_ALL_FIELD_CHOICES, MATCH_ALL_FIELD_NAME


class AdvancedSearchForm(Form):
    def __init__(self, *args, **kwargs):
        kwargs['data'] = kwargs['data'].copy()
        self.search_model = kwargs.pop('search_model')
        super().__init__(*args, **kwargs)

        fieldsets_dict = {}

        self.fields[MATCH_ALL_FIELD_NAME] = forms.ChoiceField(
            choices=MATCH_ALL_FIELD_CHOICES, widget=forms.RadioSelect,
            label=_('Match all'), help_text=_(
                'Return only results that match all fields.'
            ), required=False
        )

        for search_field in self.search_model.search_fields_label_sorted:
            if search_field.concrete and not search_field.field_name == 'id':
                self.fields[search_field.field_name] = forms.CharField(
                    help_text=search_field.get_help_text(),
                    label=search_field.label, required=False,
                    widget=forms.widgets.TextInput(
                        attrs={'type': 'search'}
                    )
                )

                # Build the fieldset dictionary.
                model = search_field.field_name_model_list[0]

                model_verbose_name_plural = model._meta.verbose_name_plural

                fieldsets_dict.setdefault(
                    model_verbose_name_plural, {
                        'fields': []
                    }
                )
                fieldsets_dict[model_verbose_name_plural]['fields'].append(
                    search_field.field_name
                )

        # Convert the fieldset dictionary to the standard fieldset tuple.
        fieldsets = ()

        fieldsets += (
            _('Search logic'), {
                'fields': (MATCH_ALL_FIELD_NAME,)
            }
        ),

        keys = list(
            fieldsets_dict.keys()
        )
        keys.sort()

        for key in keys:
            fieldsets += (
                key, fieldsets_dict[key]
            ),

        self.fieldsets = fieldsets


class SearchForm(Form):
    q = forms.CharField(
        max_length=128, label=_('Search terms'), required=False,
        widget=forms.widgets.TextInput(
            attrs={'type': 'search'}
        )
    )



from django.conf.urls import url

from .views import (
    QueueTaskTypeListView, WorkerListView, WorkerQueueListView
)

urlpatterns = [
    url(
        regex=r'^queues/(?P<queue_name>\w+)/task_types/$',
        view=QueueTaskTypeListView.as_view(), name='queue_task_type_list'
    ),
    url(
        regex=r'^workers/$', view=WorkerListView.as_view(),
        name='worker_list'
    ),
    url(
        regex=r'^workers/(?P<worker_name>\w+)/queues/$',
        view=WorkerQueueListView.as_view(), name='worker_queue_list'
    )
]



import json


class ChartRenderer:
    def __init__(self, data):
        self.data = data

    def get_chart_context(self):
        raise NotImplementedError


class RendererChartJS(ChartRenderer):
    template_name = 'statistics/renderers/chartjs/base.html'


class RendererChartJSLine(RendererChartJS):
    dataset_palette = (
        {
            'backgroundColor': 'rgba(24, 188, 156, 0.1)',
            'borderColor': '#18bc9c',
            'pointBorderWidth': 3,
            'pointHitRadius': 6,
            'pointHoverRadius': 7,
            'pointRadius': 6
        },
    )

    def get_chart_context(self):
        labels = []
        datasets = []

        for count, serie in enumerate(iterable=self.data['series'].items()):
            series_name, series_data = serie
            dataset_labels = []
            dataset_values = []

            for data_point in series_data:
                dataset_labels.extend(
                    data_point.keys()
                )
                dataset_values.extend(
                    data_point.values()
                )

            labels = dataset_labels
            dataset = {
                'cubicInterpolationMode': 'monotone',
                'data': dataset_values,
                'fill': True,
                'label': series_name
            }
            dataset.update(
                RendererChartJSLine.dataset_palette[
                    count % len(RendererChartJSLine.dataset_palette)
                ]
            )

            datasets.append(dataset)

        data = {
            'datasets': datasets,
            'labels': labels
        }

        return {
            'data': json.dumps(obj=data), 'type': 'line'
        }


class RendererChartJSDoughnut(RendererChartJS):
    chart_type = 'doughnut'
    dataset_palette = (
        {
            'backgroundColor': (
                'red', 'orange', 'yellow', 'green', 'blue', 'violet'
            )
        },
    )

    def get_chart_context(self):
        labels = []
        datasets = []

        for count, serie in enumerate(iterable=self.data['series'].items()):
            dataset = {
                'data': []
            }

            series_name, series_data = serie

            for entry in series_data:
                labels.append(
                    entry['label']
                )
                dataset['data'].append(
                    entry['value']
                )

            dataset.update(
                self.dataset_palette[
                    count % len(self.dataset_palette)
                ]
            )

            datasets.append(dataset)

        data = {
            'datasets': datasets,
            'labels': labels
        }

        return {
            'data': json.dumps(obj=data), 'type': self.chart_type
        }


class RendererChartJSPie(RendererChartJSDoughnut):
    chart_type = 'pie'
